export class ArrayBufferStore {
    constructor() {
        this._store = new Map();
    }
    clone(id) {
        const arrayBuffer = this._store.get(id);
        if (arrayBuffer === undefined) {
            throw new Error(`There is no arrayBuffer stored with an id called "${id}".`);
        }
        return arrayBuffer.slice(0);
    }
    purge(id) {
        const arrayBufferExisted = this._store.delete(id);
        if (!arrayBufferExisted) {
            throw new Error(`There is no arrayBuffer stored with an id called "${id}".`);
        }
    }
    slice(id, begin, end) {
        const arrayBuffer = this._store.get(id);
        if (arrayBuffer === undefined) {
            throw new Error(`There is no arrayBuffer stored with an id called "${id}".`);
        }
        if (begin < 0 || begin > arrayBuffer.byteLength) {
            throw new Error(`The given value for begin "${begin}" is out of bounds.`);
        }
        if (end !== null) {
            if (end > arrayBuffer.byteLength) {
                throw new Error(`The given value for end "${end}" is out of bounds.`);
            }
            if (end < begin) {
                throw new Error(`The given value for end "${end}" is below the given value for begin "${begin}".`);
            }
        }
        return arrayBuffer.slice(begin, end === null ? arrayBuffer.byteLength : end);
    }
    store(id, arrayBuffer) {
        if (this._store.has(id)) {
            throw new Error(`There is already an arrayBuffer stored with an id called "${id}".`);
        }
        this._store.set(id, arrayBuffer);
    }
}
//# sourceMappingURL=array-buffer-store.js.map